/**
 * Define api of apeman-api-watch
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const defaults = require('defaults')
const awatch = require('awatch')
const asleep = require('asleep')
const aslogger = require('aslogger')
const describe = require('./describe')
const uuid = require('uuid')
const apemanApiSSE = require('apeman-api-sse')
const apemanApiFS = require('apeman-api-fs')

/** @lends define */
function define (options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  let { root } = defaults(options, {
    root: process.cwd()
  })

  let fs = apemanApiFS({ root, logging: false })
  let sse = apemanApiSSE({ logging: false })

  let watches = {}
  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    start (pattern) {
      let id = uuid.v4()
      return co(function * () {
        let filenames = yield fs.glob(pattern, {})
        yield sse((client) => co(function * () {
          watches[ id ] = new Date()
          client.send('start', { id })
          awatch(filenames, (event, filename) => {
            client.send({
              event: 'change',
              data: { event, filename }
            })
          })
          while (watches[ id ]) {
            yield asleep(10)
          }
          client.send('stop', { id })
        }))
        return id
      })
    },

    list () {
      return co(function * () {
        return Object.assign({}, watches)
      })
    },

    stop (id) {
      return co(function * () {
        delete watches[ id ]
      })
    },

    stopAll () {
      const s = this
      return co(function * () {
        for (let id of Object.keys(watches)) {
          s.stop(id)
        }
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()
  }
}

module.exports = define
