/**
 * Define api of apeman-api-watch
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict';

var co = require('co');
var defaults = require('defaults');
var awatch = require('awatch');
var asleep = require('asleep');
var aslogger = require('aslogger');
var describe = require('./describe');
var uuid = require('uuid');
var apemanApiSSE = require('apeman-api-sse');
var apemanApiFS = require('apeman-api-fs');

/** @lends define */
function define() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var logger = aslogger({ disabled: !options.logging });

  var _defaults = defaults(options, {
    root: process.cwd()
  });

  var root = _defaults.root;


  var fs = apemanApiFS({ root: root, logging: false });
  var sse = apemanApiSSE({ logging: false });

  var watches = {};
  return {
    ping: function ping() {
      var pong = arguments.length <= 0 || arguments[0] === undefined ? 'pong' : arguments[0];

      return co(regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                logger.trace('pong:', pong);
                return _context.abrupt('return', pong);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
    },
    start: function start(pattern) {
      var id = uuid.v4();
      return co(regeneratorRuntime.mark(function _callee3() {
        var filenames;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return fs.glob(pattern, {});

              case 2:
                filenames = _context3.sent;
                _context3.next = 5;
                return sse(function (client) {
                  return co(regeneratorRuntime.mark(function _callee2() {
                    return regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            watches[id] = new Date();
                            client.send('start', { id: id });
                            awatch(filenames, function (event, filename) {
                              client.send({
                                event: 'change',
                                data: { event: event, filename: filename }
                              });
                            });

                          case 3:
                            if (!watches[id]) {
                              _context2.next = 8;
                              break;
                            }

                            _context2.next = 6;
                            return asleep(10);

                          case 6:
                            _context2.next = 3;
                            break;

                          case 8:
                            client.send('stop', { id: id });

                          case 9:
                          case 'end':
                            return _context2.stop();
                        }
                      }
                    }, _callee2, this);
                  }));
                });

              case 5:
                return _context3.abrupt('return', id);

              case 6:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
    },
    list: function list() {
      return co(regeneratorRuntime.mark(function _callee4() {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                return _context4.abrupt('return', Object.assign({}, watches));

              case 1:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));
    },
    stop: function stop(id) {
      return co(regeneratorRuntime.mark(function _callee5() {
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                delete watches[id];

              case 1:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));
    },
    stopAll: function stopAll() {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee6() {
        var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, id;

        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context6.prev = 3;

                for (_iterator = Object.keys(watches)[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  id = _step.value;

                  s.stop(id);
                }
                _context6.next = 11;
                break;

              case 7:
                _context6.prev = 7;
                _context6.t0 = _context6['catch'](3);
                _didIteratorError = true;
                _iteratorError = _context6.t0;

              case 11:
                _context6.prev = 11;
                _context6.prev = 12;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 14:
                _context6.prev = 14;

                if (!_didIteratorError) {
                  _context6.next = 17;
                  break;
                }

                throw _iteratorError;

              case 17:
                return _context6.finish(14);

              case 18:
                return _context6.finish(11);

              case 19:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this, [[3, 7, 11, 19], [12,, 14, 18]]);
      }));
    },


    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before: function $before(methodName, params) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee7() {
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                state.invokedAt = new Date();
                logger.info(methodName + ' started...');

              case 2:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after: function $after(methodName, params, returns) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee8() {
        var took;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                took = new Date() - state.invokedAt;

                logger.info('...' + methodName + ' done! (' + took + 'ms)');

              case 2:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));
    },

    /**
     * Method specifications
     */
    $spec: describe()
  };
}

module.exports = define;
//# sourceMappingURL=data:application/json;base64,bnVsbA==